using System;
using Mono.Data.SqliteClient;
using System.Diagnostics;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Timers;
using System.Threading;

namespace Monocular {
	class Monocular {
		static bool FirstTime = false;
		static SqliteConnection Conn;

		static void Main(string[] args) {
			if (!File.Exists("monocular.db")) {
				FirstTime = true;
				File.Create("monocular.db");
			}

			Conn = new SqliteConnection("URI=file:monocular.db");
			Conn.Open();

			SqliteCommand dbcmd = Conn.CreateCommand();

			if (FirstTime) {
				dbcmd.CommandText = "CREATE TABLE processes (DateStamp TEXT, Name TEXT, Id INTEGER);";
				dbcmd.ExecuteNonQuery();
			}

			if (args.Length != 0) {
				// we're reading the data

				switch (args[0]) {
					case "id":
						dbcmd.CommandText = "SELECT DateStamp, Name, Id FROM processes ORDER BY DateStamp, Id";
						break;

					case "name":
						dbcmd.CommandText = "SELECT DateStamp, Name, Id FROM processes ORDER BY DateStamp, Name";
						break;

					default:
						Console.WriteLine("Please specify either 'id' or 'name'.");
						return;
				}

				string currenttime = "";

				SqliteDataReader reader = dbcmd.ExecuteReader();
				while (reader.Read()) {
					string datestamp = reader.GetString(0);
					string name = reader.GetString(1);
					int id = reader.GetInt32(2);
					if (datestamp != currenttime) {
						Console.WriteLine("");
						Console.WriteLine("Readings from " + datestamp);
						Console.WriteLine(" PID\tName");
						currenttime = datestamp;
					}

					Console.WriteLine(" {0}\t{1}\t", id, name);
				}

				reader.Close();
				reader = null;


				return;
			}

			dbcmd.Dispose();
			dbcmd = null;

			System.Timers.Timer timer = new System.Timers.Timer();
			timer.Interval = 10000;
			timer.Enabled = true;

			timer.Elapsed += new ElapsedEventHandler(TimesUp);

			TimesUp(null, null);

			Thread.Sleep(Timeout.Infinite);
		}

		static void TimesUp(object sender, ElapsedEventArgs e) {
			Process[] proclist = Process.GetProcesses();

			SqliteTransaction transaction = (SqliteTransaction)Conn.BeginTransaction();
			SqliteCommand dbcmd = Conn.CreateCommand();
			dbcmd.CommandText = "INSERT INTO processes VALUES (?, ?, ?);";

			SqliteParameter date = new SqliteParameter();
			SqliteParameter name = new SqliteParameter();
			SqliteParameter id = new SqliteParameter();

			date.Value = DateTime.Now.ToLongTimeString();

			dbcmd.Parameters.Add(date);
			dbcmd.Parameters.Add(name);
			dbcmd.Parameters.Add(id);

			foreach (Process proc  in proclist) {
				name.Value = proc.ProcessName;
				id.Value = proc.Id;

				dbcmd.ExecuteNonQuery();
			}


			dbcmd.Dispose();
			dbcmd = null;

			transaction.Commit();
		}
	}
}